//////////////////////////////////////////////////////////////////////////////////////////
// Project description
// 
// Name: Touchlib barrel distortion correction tool
// Purpose: 
// - Correcting the barrel distortion of the lens
//
// Laurence Muller (aka Falcon4ever)
// Contact: info@falcon4ever.com
// Date: 19/12/2007
// Site: www.multigesture.net
// Licensetype: Same as the original sample of OpenCV
//
// Read main.cpp for project details
//////////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_WARNINGS
#include <string>
#include <stdio.h>
#include <time.h>
#include <conio.h>

// OpenCV stuff
#include <cv.h>
#include <highgui.h>
#ifdef WIN32
	#pragma comment(lib, "cv.lib")
	#pragma comment(lib, "cxcore.lib")
	#pragma comment(lib, "highgui.lib")
#endif

// Take from Touchlib's CMU input filter
#include <1394Camera.h>
#ifdef WIN32
	#pragma comment(lib, "1394camera.lib")
#endif

#define CMU160x120yuv444 "160x120yuv444"
#define CMU320x240yuv422 "320x240yuv422"
#define CMU640x480yuv411 "640x480yuv411"
#define CMU640x480yuv422 "640x480yuv422"
#define CMU640x480rgb "640x480rgb"
#define CMU640x480mono "640x480mono"
#define CMU640x480mono16 "640x480mono16"
#define CMU800x600yuv422 "800x600yuv422"
#define CMU800x600rgb "800x600rgb"
#define CMU800x600mono "800x600mono"
#define CMU1024x768yuv422 "1024x768yuv422"
#define CMU1024x768rgb "1024x768rgb"
#define CMU1024x768mono "1024x768mono"
#define CMU800x600mono16 "800x600mono16"
#define CMU1024x768mono16 "1024x768mono16"
#define CMU1280x960yuv422 "1280x960yuv422"
#define CMU1280x960rgb "1280x960rgb"
#define CMU1280x960mono "1280x960mono"
#define CMU1600x1200yuv422 "1600x1200yuv422"
#define CMU1600x1200rgb "1600x1200rgb"
#define CMU1600x1200mono "1600x1200mono"
#define CMU1280x960mono16 "1280x960mono16"
#define CMU1600x1200mono16 "1600x1200mono16"

#define CMURate2fps "2fps"
#define CMURate4fps "4fps"
#define CMURate7fps "7fps"
#define CMURate15fps "15fps"
#define CMURate30fps "30fps"
#define CMURate60fps "60fps"
#define CMURate120fps "120fps"
#define CMURate240fps "240fps"

// Calibration tool (most functions are taken from OpenCV's sample calibration.cpp)
enum { DETECTION = 0, CAPTURING = 1, CALIBRATED = 2 };
double compute_reprojection_error(	const CvMat* object_points,
									const CvMat* rot_vects, 
									const CvMat* trans_vects,
									const CvMat* camera_matrix, 
									const CvMat* dist_coeffs,
									const CvMat* image_points, 
									const CvMat* point_counts,
									CvMat* per_view_errors );

int run_calibration(	CvSeq* image_points_seq, 
						CvSize img_size, 
						CvSize board_size,
						float square_size, 
						float aspect_ratio, 
						int flags,
						CvMat* camera_matrix, 
						CvMat* dist_coeffs, 
						CvMat** extr_params,
						CvMat** reproj_errs, 
						double* avg_reproj_err );

void save_camera_params(const CvMat* camera_matrix, CvMat* dist_coeffs);

// Misc stuff
FILE* f = 0;
char imagename[1024];
CvSize board_size = {0,0};
int i;
int image_count = 10;
int delay = 1000;
float square_size = 1.f;
int interface_type;
int elem_size;
CvPoint2D32f* image_points_buf = 0;
CvMemStorage* storage;
CvSeq* image_points_seq = 0;
int flip_vertical = 0;
CvSize img_size = {0,0};
int mode = DETECTION;
clock_t prev_timestamp = 0;
CvFont font = cvFont( 1, 1 );
double _camera[9], _dist_coeffs[4];
CvMat camera = cvMat( 3, 3, CV_64F, _camera );
CvMat dist_coeffs = cvMat( 1, 4, CV_64F, _dist_coeffs );
CvMat *extr_params = 0, *reproj_errs = 0;
int undistort_image = 0;
float aspect_ratio = 1.f;
int flags = 0;
double avg_reproj_err = 0;

// Used for USB camera's
CvCapture* capture = 0;

// FIREWIRECAM HACK :P
// Firewire camera functions
int cmu_init();
void cmu_setFeature(CAMERA_FEATURE id, int valueL, int valueH);

C1394Camera* cmu_capture = 0;	
unsigned long cmu_w, cmu_h;
IplImage * cmu_rawImage;
bool cmu_flipRGB;
int cmu_mode;
int cmu_format;
int cmu_rate;
bool cmu_mono;

std::string cmu_sMode, cmu_sRate, cmu_sVendor, cmu_sName;
LARGE_INTEGER cmu_lID;

// camera features
int cmu_sharpness;
int cmu_gain;
int cmu_brightness;
int cmu_exposure;
int cmu_whitebalanceL, cmu_whitebalanceH;
int cmu_hue;
int cmu_saturation;
int cmu_gamma;

IplImage* undistorted_with_border( const IplImage *image, const CvMat *intrinsic,const CvMat *distortion, short int border );
